module initMicThompson

	real,allocatable :: dataValues(:,:,:,:,:)
	character(len=256), allocatable :: varName(:)
	integer :: xdef,ydef,zdef,zbeg,zend,tdef,vars
	real :: xbeg,xstep,ybeg,ystep

	real, allocatable :: qnifa(:,:,:,:) !T,Lev,i,j
	real, allocatable :: qnwfa(:,:,:,:) !T,Lev,i,j
	real,parameter,dimension(30) :: press_mb = (/&
	    	2013. , 2001. , 1000. , 975. , 950. &
	       , 925. ,  900. ,  875. , 850. , 825. &
	       , 800. ,  775. ,  750. , 700. , 650. &
	       , 600. ,  550. ,  500. , 450. , 400. &
	       , 350. ,  300. ,  250. , 225. , 200. &
	       , 175. ,  150. ,  125. , 100. ,  50. &
	       /)

	integer :: pastMonth

	contains

	subroutine readDataFriendly()
		  !# Read aerosol friendly information for ice and water
      !#
      !# @note
      !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
      !#
      !# *Read aerosol friendly information for ice and water
      !#
      !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
      !#
      !# **Author**: Luiz Flavio Rodrigues **&#9993;**<luiz.rodrigues@inpe.br>
      !#
      !# **Date**: 07 May 2019 (Tuesday)
      !# @endnote
      !#
      !# @changes
      !#
      !# +
      !# @endchanges
      !# @bug
      !#
      !#@endbug
      !#
      !#@todo
      !#  &#9744; <br/>
      !# @endtodo
      !#
      !# @warning
      !# Now is under CC-GPL License, please see
      !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
      !# @endwarning
      !#
      !#---
		use node_mod, only: &
     		nodei0, & ! intent(in)
     		nodej0, & ! intent(in)
     		nodemxp, & ! intent(in)
     		nodemyp, & ! intent(in)
     		nmachs,  & ! intent(in)
     		mynum,  &  ! intent(in)
     		mchnum, &  ! intent(in)
     		master_num, &! intent(in)
     		ia,iz,ja,jz

     	use dump, only: &
     		dumpMessage

		use ReadBcst, only: & !Just for broadcast comm infos
    		Broadcast

    	use parlib, only: &
         	parf_bcast ! Subroutine

	    implicit none
	
	    include "constants.f90"
	    include "i8.h"
	
	    !Parameters (constants)
	    character(len=*), parameter :: header="**(readDataFriendly)**"
	    character(len=*), parameter :: fileDir='./tables/micro'
	    character(len=*), parameter :: fileCtl=fileDir//'/friend_data.ctl'
	    integer, parameter :: fNum=33

	
	    ! Input/Output variables
	
	    !Local variables
	    character(len=256) :: lixo
	    character(len=256) :: dset
	    character(len=256) :: title
	    real :: undef
	    real, allocatable :: vvar(:,:)
	    logical :: fileExist
	    integer :: recSize
	    integer :: rec,nVar,nTime,nLev


	    !Code
	    pastMonth=0

	    if(mchnum==master_num) then

	    	iErrNumber=dumpMessage(c_tty,c_yes,header,modelVersion &
            ,c_notice,'Reading fliendly ice and water aerosols')

      		inquire(file=trim(filectl), exist=fileExist )
    		if ( .not. fileExist ) iErrNumber=dumpMessage(c_tty,c_yes,header,modelVersion &
            ,c_fatal,'file '//trim(filectl)//' not found. Please, check it!')

	    	open (unit=fNum, file=trim(filectl), status='old' &
	    		, access='sequential', form='formatted', action='read' )
   			read(fNum, *) lixo,dset  !'DSET ^friend_data.dat'
   			read(fNum, *) lixo,title !'TITLE QNWFA_QNIFA_SIGMA_MONTHLY_.dat '
   			read(fNum, *) lixo,undef !'UNDEF -9.99E33'
   			read(fNum, *) lixo,xdef,lixo,xbeg,xstep!'XDEF 288 LINEAR -180 1.25'
   			read(fNum, *) lixo,ydef,lixo,ybeg,ystep !'YDEF 181 LINEAR -90 1.0'
   			read(fNum, *) lixo,zdef,lixo,zbeg,zend !'ZDEF 30 LINEAR 1 1'
   			read(fNum, *) lixo,tdef !'TDEF 12 LINEAR 00z01Jan2007 1mo'
   			read(fNum, *) lixo,vars !'VARS 3'
	
   			if(.not. allocated(dataValues)) then
   				allocate(dataValues(vars,tdef,zdef,xdef,ydef))
   				allocate(varName(vars))
   			endif
	
   			do nVar=1,vars
	   			read(fNum, *) varName(nVar)
	   		enddo
   			close(fNum)


   			dset=trim(fileDir)//'/'//trim(dset(2:))
      		inquire(file=trim(dset), exist=fileExist )
    		if ( .not. fileExist ) iErrNumber=dumpMessage(c_tty,c_yes,header,modelVersion &
            ,c_fatal,'file '//trim(dset)//' not found. Please, check it!')

            recSize=xdef*ydef*4
   			open (fNum,FILE=trim(dSet),FORM='UNFORMATTED' &
   				  ,STATUS='OLD',ACCESS='DIRECT',RECL=recSize, action='read' )

   			rec=1
   			do nTime=1,tdef
   			   do nVar=1,vars
   			      do nLev=1,zdef
   			         read (fNum,REC=rec) dataValues(nvar,nTime,nLev,:,:)
   			         rec = rec + 1
   			      enddo
   			   enddo
   			enddo

   			close(fNum)

   		endif

   		!Send dimensions to all slaves
   		call Broadcast(vars, master_num, "vars")
   		call Broadcast(tdef, master_num, "tdef")
   		call Broadcast(zdef, master_num, "zdef")
   		call Broadcast(xdef, master_num, "xdef")
   		call Broadcast(ydef, master_num, "ydef")

   		if(mchnum/=master_num) then
   			allocate(dataValues(vars,tdef,zdef,xdef,ydef))
   			allocate(varName(vars))
   		endif
   		allocate(vvar(xdef,ydef))
  
  		!Send values to all slaves
  		do ntime=1,tdef
    		do nVar=1,vars
      			do nLev=1,zdef
        			vvar=dataValues(nvar,nTime,nLev,:,:)
        			call parf_bcast(vvar,int(xdef,i8), int(ydef,i8) &
           			,master_num)
        			dataValues(nvar,nTime,nLev,:,:)=vvar
      			enddo
    		enddo 
  		enddo

  		call interpolateDataFriendly()

	end subroutine readDataFriendly

	subroutine interpolateDataFriendly()
      !# Interpolate the data from firendly aerosol do model's grid
      !#
      !# @note
      !# ![](http://brams.cptec.inpe.br/wp-content/uploads/2015/11/logo-brams.navigation.png "")
      !#
      !# **Brief**: Interpolate the data from firendly aerosol do model's grid
      !# use the formula of inverse quadratic distance to interpolate:
      !# {{\sum_{i=1}^{n}({1\over{d_i^2}}X_i)}}\over{{\sum_{i=1}^{n}({1\over{d_i^2}})}}
      !#
      !# **Documentation**: <http://brams.cptec.inpe.br/documentation/>
      !#
      !# **Author**: Luiz Flavio Rodrigues **&#9993;**<luiz.rodrigues@inpe.br>
      !#
      !# **Date**: 07 May 2019 (Tuesday)
      !# @endnote
      !#
      !# @changes
      !#
      !# +
      !# @endchanges
      !# @bug
      !#
      !#@endbug
      !#
      !#@todo
      !#  &#9744; <br/>
      !# @endtodo
      !#
      !# @warning
      !# Now is under CC-GPL License, please see
      !# &copy; <https://creativecommons.org/licenses/GPL/2.0/legalcode.pt>
      !# @endwarning
      !#
      !#---
	    use node_mod, only: &
     		nodei0, & ! intent(in)
     		nodej0, & ! intent(in)
     		nodemxp, & ! intent(in)
     		nodemyp, & ! intent(in)
     		nmachs,  & ! intent(in)
     		mynum,  &  ! intent(in)
     		mchnum, &  ! intent(in)
     		master_num, &! intent(in)
     		ia,iz,ja,jz

     	use dump, only: &
     		dumpMessage

  		use mem_grid, only : &
  			grid_g, &
        zt, &
     		runtype,  &  ! intent(in)
     		iyear1, &    ! intent(in)
     		imonth1, &   ! intent(in)
     		idate1, &    ! intent(in)
     		itime1, &    ! intent(in)
     		nnxp, &      ! intent(in)
     		nnyp, &      ! intent(in)
     		nnzp,    &   ! intent(in)
     		GlobalSizes  ! Subroutine

     	use readbcst, only: &
     		gatherData   ! Subroutine

     	use mem_basic, only: &
      		basic_g

      use dam, only: &
          getDistance

	    implicit none
	
	    include "constants.f90"
	
	    !Parameters (constants)
	    character(len=*), parameter :: header="**(interpolateDataFriendly)**"
	    integer, parameter :: ifm=1
      !# Grid number
      integer, parameter :: nMaxLatLons=20
      !# Maximum points to distance 
      integer, parameter :: nifa=1
      integer, parameter :: nwfa=2
      !# Id of data inside arrays
      logical, parameter :: byQuadratic=.false.
      !# If vertical interpolation is using inverse of quadratic distance
      logical, parameter :: dumpGrads=.false.
      !# Create a grads file: use only with one processor
	
	    !Local variables
      real(kind=kind_rb) :: sumIfa
      real(kind=kind_rb) :: sumWfa
      real(kind=kind_rb) :: sumY
      real(kind=kind_rb) :: distance
      real(kind=kind_rb) :: mLat
      real(kind=kind_rb) :: mLon
      real :: MaxLats
      real :: MaxLons
      real(kind=kind_rb),allocatable :: Glat(:)
      real(kind=kind_rb),allocatable :: Glon(:)
      real :: pressBrams(nnzp(ifm))
      real :: picpi
      real :: invDistanceQuad

      integer :: t,i,j,ii,jj,k,kk,recordLen,iRec,funit
      real,allocatable :: qnifaLocal(:,:,:,:)
      real,allocatable :: qnWfaLocal(:,:,:,:)
      real,allocatable :: hModel(:,:,:)
      real,allocatable :: hFriendly(:)

  		if(mchnum==master_num) &
  		   iErrNumber=dumpMessage(c_tty,c_yes,header,modelVersion &
            ,c_notice,'Interpolate fliendly ice and water aerosols')

      !Allocating the list of lons and lats in friendly aerosol
      allocate(Glon(xdef),Glat(ydef))

      allocate(hModel(nnzp(ifm),nodemxp(mynum,ifm) &
        ,nodemyp(mynum,ifm)))
      allocate(hFriendly(zdef))
      !
      !Allocating the 2 variables qnifa and qnwfa with model's levels
      allocate(qnifa(tdef,nnzp(ifm),nodemxp(mynum,ifm) &
        ,nodemyp(mynum,ifm))) 
      allocate(qnwfa(tdef,nnzp(ifm),nodemxp(mynum,ifm) &
        ,nodemyp(mynum,ifm)))
      !
      !Allocating the 2 variables qnifa and qnwfa with friendly aerosol levels
      allocate(qnifaLocal(tdef,zdef,nodemxp(mynum,ifm) &
        ,nodemyp(mynum,ifm))) 
      allocate(qnwfaLocal(tdef,zdef,nodemxp(mynum,ifm) &
        ,nodemyp(mynum,ifm)))

      !Limiting the space around the points to interpolate
      !The code will compute nMaxLatLons points with deltaLat and
      !deltaLon size in each direction 
      maxLats=(grid_g(ifm)%glat(1,2)-grid_g(ifm)%glat(1,1))*nMaxLatLons
      maxLons=(grid_g(ifm)%glon(2,1)-grid_g(ifm)%glon(1,1))*nMaxLatLons

      !Fill the list of lats & lons from friendly aerosol
      !Consider equal distances (delta) in all latitudes&long.
      do i=1,xdef
        Glon(i)=xbeg+(i-1)*xstep
      enddo
      do j=1,ydef
        Glat(j)=ybeg+(j-1)*ystep
      enddo
	
      ! Calc the value of nifa (ice friendly) and nwfa (water friendly)
      ! using inverse of quadratic distance between each model's point 
      ! position and the friendly aerosol global point position.
      !
      do t=1,tdef
        do k=1,zdef
          do i=ia,iz
            do j=ja,jz
              ! if(i==10 .and. j==10) then
              !     write (23,fmt='(4(A3,1X),4(A8,1X),8(A18,1X))') &
              !                   't','k','ii','jj','mLat','mLon','Glat(jj)','Glon(ii)' &
              !                   ,'distance','nifa' &
              !                   ,'nwfa','sumIfa','sumWfa','sumY','Ifa','Wfa'
              ! endif
              sumIfa=0.0
              sumWfa=0.0
              sumY=0.0
              mLat=grid_g(ifm)%glat(i,j)
              mLon=grid_g(ifm)%glon(i,j)
              do ii=1,xdef
                do jj=1,ydef
                  !
                  !If is out of limits dont process the point
                  if(Glon(ii)<mLon-maxLons .or. &
                     Glon(ii)>mLon+maxLons .or. &
                     Glat(jj)<mLat-maxLats .or. &
                     Glat(jj)>mLat+maxLats) cycle
                  !
                  !Distance between model point lat/lon to friendly 
                  ! lat/lon point
                  distance=getDistance(Glat(jj),Glon(ii),mLat,mLon)
                  invDistanceQuad=1.0/(distance*distance)
                  !Using the inverse quadratic method to sum vars
                  sumIfa=sumIfa+ invDistanceQuad&
                         *dataValues(nifa,t,k,ii,jj)
                  sumWfa=sumWfa+invDistanceQuad &
                         *dataValues(nwfa,t,k,ii,jj)
                  sumY=sumY+invDistanceQuad
                  ! if(i==10 .and. j==10) then
                  !   write (23,fmt='(4(I3.3,1X),4(F8.2,1X),8(E18.2,1X))') &
                  !               t,k,ii,jj,mLat,mLon,Glat(jj),Glon(ii) &
                  !               ,distance,dataValues(nifa,t,k,ii,jj) &
                  !               ,dataValues(nwfa,t,k,ii,jj),sumIfa,sumWfa,sumY,sumIfa/sumY,sumWfa/sumY
                  ! endif
                enddo
              enddo
              !
              !The local qnifa and qnwfa still using the
              !number of levels from friendly data. They will be
              !changed to model levels when vertical interpolation 
              !will be done on code bellow
              if(sumY/=0.0) then
                qnifaLocal(t,k,i,j)=sumIfa/sumY
                qnwfaLocal(t,k,i,j)=sumWfa/sumY
              endif
              !
            enddo
          enddo
        enddo
      enddo

if(dumpGrads) then
     open(unit=funit,file='friendly_I.ctl' &
         ,action='WRITE',status='replace',form='FORMATTED')
  
      !writing the name of grads file
      write(funit,*) 'dset ^friendly_I.gra'
      !writing others infos to ctl
      write(funit,*) 'undef -0.9990000E+34'
      write(funit,*) 'title Friendly Data'
      write(funit,*) 'xdef ',nodemxp(mynum,ifm),' linear ' &
      ,grid_g(ifm)%glon(1,1),' ',(grid_g(ifm)%glon(2,1)-grid_g(ifm)%glon(1,1))
      write(funit,*) 'ydef ',nodemyp(mynum,ifm),' linear ' &
      ,grid_g(ifm)%glat(1,1),' ',(grid_g(ifm)%glat(1,2)-grid_g(ifm)%glat(1,1))
      write(funit,*) 'zdef ',zdef,'levels ',press_mb(:)
      write(funit,*) 'tdef 12 linear 00:00z01jan2018     1mo'
      write(funit,*) 'vars 2'
      write(funit,*) 'QNIFA       ',nnzp(ifm),' 99 Ice'
      write(funit,*) 'QNWFA       ',nnzp(ifm),' 99 Water      '
      write(funit,*) 'endvars'
      close(funit)  
  
      recordLen=4*nodemxp(mynum,ifm)*nodemyp(mynum,ifm)
  
      open(unit=funit,file='friendly_I.gra',&
        action='WRITE',status='REPLACE',form='UNFORMATTED',access='DIRECT', &
        recl=recordLen)
  
      !# writing grads binary and fill variables
      irec=1
      do t=1,12
        do k=1,zdef
          write (funit,rec=irec) qnifaLocal(t,k,:,:)
          irec=irec+1
        enddo
        do k=1,zdef
          write (funit,rec=irec) qnWfaLocal(t,k,:,:)
          irec=irec+1
        enddo
      enddo
      close(funit)

    endif

      !Interpolate in vertical direction
      if(byQuadratic) then !if using inverse of distance 
        !Calc the model's altitudes
        do i=ia,iz
          do j=ja,jz
            do k=1,nnzp(ifm)
              hModel(k,i,j)= zt(k) * grid_g(ifm)%rtgt(i,j) &
                          + grid_g(ifm)%topt(i,j)
            enddo
          enddo
        enddo
        !calc the Data friendly altitudes using hypsometric formula
        !Setting the temperature in 303.15K and Sea level Press in 1000mBar.
        do k=3,zdef
          hFriendly(k)=(((1000./press_mb(k))**(1.0/5.257)-1.0)*303.15)/0.0065
        enddo
        
        do t=1,tdef
          do j=ja,jz
            do i=ia,iz  
              do k=1,nnzp(ifm)
                sumIfa=0.0
                sumWfa=0.0
                sumY=0.0
                do kk=3,zdef
                  distance=(hFriendly(kk)-hModel(k,i,j))/100.
                  sumIfa=sumIfa+(1.0/(distance*distance))*qnifaLocal(t,kk,i,j)
                  sumWfa=sumWfa+(1.0/(distance*distance))*qnwfaLocal(t,kk,i,j)
                  sumY=sumY+(1.0/distance*distance)
                enddo
                qnifa(t,k,i,j)=sumIfa/sumY
                qnwfa(t,k,i,j)=sumWfa/sumY
              enddo
            enddo
          enddo
        enddo

    else !using pressure levels

      do t=1,tdef
        do j=ja,jz
          do i=ia,iz 
            !Calc the model pressure in each layer
            do k=nnzp(ifm),1,-1
              picpi = (basic_g(ifm)%pi0(k,i,j) + basic_g(ifm)%pp(k,i,j)) * c_cpi
              pressBrams(k) = (c_p00 * picpi ** c_cpor)*c_adjust
              do kk=zdef,2,-1
                if(pressBrams(k)>=press_mb(kk) .and. pressBrams(k)<press_mb(kk-1)) then
                  !if(t==1 .and. k==1) write(28,*) i,j,kk,qnifaLocal(t,kk,i,j)
                  qnifa(t,k,i,j)=qnifaLocal(t,kk,i,j)
                  qnwfa(t,k,i,j)=qnwfaLocal(t,kk,i,j)
                endif
              enddo
            enddo
          enddo
        enddo
      enddo 
    endif
  
    ! do t=1,tdef
    !   do j=ja,jz
    !     do i=ia,iz 
    !       write(28,fmt='("Ifa: ",3(I2.2,1X),40(F18.2,1X))') t,i,j,qnifa(t,:,i,j)
    !       write(28,fmt='("Wfa: ",3(I2.2,1X),40(F18.2,1X))') t,i,j,qnwfa(t,:,i,j)
    !     enddo
    !   enddo
    ! enddo



    if(dumpGrads) then
	   open(unit=funit,file='friendly_r.ctl' &
         ,action='WRITE',status='replace',form='FORMATTED')
  
      !writing the name of grads file
      write(funit,*) 'dset ^friendly_r.gra'
      !writing others infos to ctl
      write(funit,*) 'undef -0.9990000E+34'
      write(funit,*) 'title Friendly Data'
      write(funit,*) 'xdef ',nodemxp(mynum,ifm),' linear ' &
      ,grid_g(ifm)%glon(1,1),' ',(grid_g(ifm)%glon(2,1)-grid_g(ifm)%glon(1,1))
      write(funit,*) 'ydef ',nodemyp(mynum,ifm),' linear ' &
      ,grid_g(ifm)%glat(1,1),' ',(grid_g(ifm)%glat(1,2)-grid_g(ifm)%glat(1,1))
      write(funit,*) 'zdef ',nnzp(ifm),'levels ',pressBrams(:)
      write(funit,*) 'tdef 12 linear 00:00z01jan2018     1mo'
      write(funit,*) 'vars 2'
      write(funit,*) 'QNIFA       ',nnzp(ifm),' 99 Ice'
      write(funit,*) 'QNWFA       ',nnzp(ifm),' 99 Water      '
      write(funit,*) 'endvars'
      close(funit)  
  
      recordLen=4*nodemxp(mynum,ifm)*nodemyp(mynum,ifm)
  
      open(unit=funit,file='friendly_r.gra',&
        action='WRITE',status='REPLACE',form='UNFORMATTED',access='DIRECT', &
        recl=recordLen)
  
      !# writing grads binary and fill variables
      irec=1
      do t=1,12
        do k=1,nnzp(ifm)
          write (funit,rec=irec) qnifa(t,k,:,:)
          irec=irec+1
        enddo
        do k=1,nnzp(ifm)
          write (funit,rec=irec) qnwfa(t,k,:,:)
          irec=irec+1
        enddo
      enddo
      close(funit)

    endif

	    
	end subroutine interpolateDataFriendly

	subroutine adJustFriendlyForMonth(time)
		use mem_micro, only: &
      		micro_g

      	 use mem_grid, only: &
     		idate1, &
     		imonth1, &
     		iyear1, &
     		ihour1, &
     		itime1

     	use node_mod, only: &
     		nodei0, & ! intent(in)
     		nodej0, & ! intent(in)
     		nodemxp, & ! intent(in)
     		nodemyp, & ! intent(in)
     		nmachs,  & ! intent(in)
     		mynum,  &  ! intent(in)
     		mchnum, &  ! intent(in)
     		master_num, &! intent(in)
     		ia,iz,ja,jz

     	use ModDateUtils, only: &
       		date_add_to

       	use mem_grid, only : &
  			grid_g, &
     		runtype,  &  ! intent(in)
     		iyear1, &    ! intent(in)
     		imonth1, &   ! intent(in)
     		idate1, &    ! intent(in)
     		itime1, &    ! intent(in)
     		nnxp, &      ! intent(in)
     		nnyp, &      ! intent(in)
     		nnzp,    &   ! intent(in)
     		GlobalSizes  ! Subroutine

	    implicit none
	
	    include "constants.f90"
	
	    !Parameters (constants)
		integer, parameter :: ifm=1

	    ! Input/Output variables
	    real   ,intent(in)    :: time
	    !# current time
	
	    !Local variables
		integer :: oyr,omn,ody,otm
		integer :: i,j,k
		

	    !Code
	    call date_add_to(iyear1,imonth1,idate1,itime1 &
	    	     *100,time,'s',oyr,omn,ody,otm)

	    !print *,'omn=',omn
	    !print *,associated(micro_g(ifm)%cccnp),associated(micro_g(ifm)%cifnp)
	    !call flush(6)

	    if(omn==pastMonth) return

	    pastMonth=omn

	    micro_g(ifm)%cccnp(1:nnzp(ifm),ia:iz,ja:jz)= &
	        qnifa(omn,1:nnzp(ifm),ia:iz,ja:jz)
        micro_g(ifm)%cifnp(1:nnzp(ifm),ia:iz,ja:jz)= &
            qnwfa(omn,1:nnzp(ifm),ia:iz,ja:jz)
	    
	end subroutine adJustFriendlyForMonth


end module initMicThompson